/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann		   *
 *      																   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "LuaController.h"

// bool for removing the hook
bool LuaController::quit = false;

/**
 * \brief Calls init().
 *
 * \param parent Pointer to the parent QObject.
 * \see init()
 */
LuaController::LuaController ( QObject* parent ) : Controller(parent)
{
	setName("LuaController");
    init ();
}

/**
 * \brief Closes the referenced Lua instance.
 */
void LuaController::close ()
{
    lua_close ( L_ );
}

/**
 * \brief Executes a String with Lua.
 *
 * This method allows the execution of short Lua code fragments, with the
 * lua_State referenced by attribute L_. lstop() is used as a hook and
 * stop() allows to cancel the execution.
 *
 * \param luacode QString with the Lua code to execute.
 * \return Lua error code.
 * \see stop()
 * \see lstop()
 * \see error()
 * \see endExecution()
 */
int LuaController::doString ( QString& luacode )
{
    emit startExecution();

    // set lstop as hook to cancel execution if necessary
    lua_sethook ( L_, &LuaController::lstop, LUA_MASKLINE | LUA_MASKCALL
                                           | LUA_MASKRET  | LUA_MASKCOUNT, 1 );

    // execute string
    int status = luaL_dostring( L_,luacode.toLatin1().data() );

    // check for errors and emit signal with error description if any
    if ( status != 0 )
    {
        emit error( reportErrors ( status ) );
    }

    // let other classes know that execution is ready
    emit endExecution();

    return status;
}

/**
 * \brief Executes a file with Lua.
 *
 * This method allows the execution of Lua code from a file. The execution will
 * alter the lua_State referenced by attribute L_. lstop() is used as a hook and
 * stop() allows to cancel the execution.
 *
 * \param file QString with path to the file to execute.
 * \return Lua error code.
 * \see stop()
 * \see lstop()
 * \see error()
 * \see endExecution()
 */
int LuaController::doFile ( QString file )
{
    emit startExecution();
    // set lstop as hook to cancel execution if necessary
    lua_sethook ( L_, &LuaController::lstop, LUA_MASKLINE | LUA_MASKCALL
                                           | LUA_MASKRET  | LUA_MASKCOUNT, 1 );

    // execute file
    int status = luaL_dofile ( L_, file.toLatin1().data() );

    // check for errors and emit signal with error description if any
    if ( status != 0 )
    {
        emit error ( reportErrors ( status ) );
    }

    // let UCQConsole know execution is ready
    emit endExecution();
    return status;
}

/**
 * \brief Gets a list with all functions available in the currently referenced
 *        lua_State object.
 *
 * This method retrieves a list of the names of all functions, with a common
 * prefix, that are available in the current lua_State. It puts the names in
 * the QStringList referenced by the list argument.
 *
 * \param prefix Common prefix of the functions to get.
 * \param list Reference to the list that should hold the function names after
 *             the call.
 * \see UCQConsole::actionKeyComplete()
 */
void LuaController::getCompletionList ( const QString& prefix,
                                        QStringList& list )
{
    // get all defined functions
    lua_getglobal ( L_, "functions" );
    lua_call ( L_, 0, 1 );

    // split the names up
    QString comp ( lua_tostring( L_, -1 ) );
    list = comp.split ( '\n' );

    // find the functions with the desired prefix
    list = list.filter ( QRegExp ( "^" + prefix ) );
    list.sort ();

    // remove the list from the stack
    lua_pop(L_, 1);
}

/**
 * \brief Initializes the lua_State and loads all Lua base libraries into it.
 */
void LuaController::init ()
{
    // initialize Lua
    L_ = lua_open();

    // load Lua base libraries
    luaL_openlibs ( L_ );
}

/**
 * \brief Lua_Hook that checks, if quit_ is true and ends the current execution
 *        in that case.
 *
 * \param L The instance of Lua.
 * \see stop()
 */
void LuaController::lstop ( lua_State *L, lua_Debug * )
{
    if ( LuaController::quit )
    {
        LuaController::quit = false;

        // stop current exection by removing the hook
        lua_sethook ( L, NULL, 0, 0 );

        // emit an error signal
        luaL_error ( L, tr ( "Interrupted." ).toAscii () );
    }
}

/**
 * \brief Send print signal with the given text.
 *
 * This function emits the printSignal to indicate, that a QString is ready for
 * printing.
 *
 * \param text Text to print.
 * \see LuaOutput::print()
 * \see UCQConsole
 */
void LuaController::print ( QString text )
{
    emit printSignal ( text );
}

/**
 * \brief Returns the description of a Lua error code.
 *
 * \param status Lua error code.
 * \return Description of the error code.
 * \see doString()
 * \see doFile()
 */
QString LuaController::reportErrors ( const int status )
{
    if ( status != 0 )
    {
        // get error description
        QString error ( lua_tostring(L_, -1) );

        // remove error from stack
        lua_pop(L_, 1);

        return error;
    }
    return "";
}

/**
 * \brief Stops any currently running execution.
 *
 * This method sets the quit_ attribute to true, to stop the current execution
 * with lstop().
 *
 * \see lstop()
 */
void LuaController::stop ()
{
    LuaController::quit = true;
}
