/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "TableController.h"

/**
 * \brief Sets the TableModel and connects the Signals to it.
 *
 * \param parent Pointer to the  parent QObject.
 * \param model The tableModel to use.

 */
TableController::TableController ( QObject* parent, TableModel* model ) :
    Controller ( parent )
{
	setName("TableController");

    model_ = model;
    qRegisterMetaType<TableModel::CollationModes> (
                                                    "TableModel::CollationModes" );
    qRegisterMetaType<QList<int> > ( "QList<int>" );

    connect ( this,   SIGNAL ( clear ()        ),
              model_, SLOT   ( clearVisible () ),
              Qt::BlockingQueuedConnection );

    connect ( this,   SIGNAL ( selectBlocksSignal ( QStringList, bool ) ),
              model_->getBlockList (),
                      SLOT   ( selectBlocks ( QStringList, bool ) ),
              Qt::BlockingQueuedConnection );

    connect ( this,   SIGNAL ( addRangeSignal ( uint, uint ) ),
              model_, SLOT   ( addRange       ( uint, uint ) ),
              Qt::BlockingQueuedConnection );

    connect ( this,   SIGNAL ( hideRangeSignal ( uint, uint ) ),
              model_, SLOT   ( hideRange       ( uint, uint ) ),
              Qt::BlockingQueuedConnection );

    connect ( this, SIGNAL ( addWeightRangeSignal ( QStringList, QStringList,
                                    TableModel::CollationModes, QList<int> ) ),
              model_, SLOT( addWeightRange        ( QStringList, QStringList,
                                    TableModel::CollationModes, QList<int> ) ),
              Qt::BlockingQueuedConnection );
}

/**
 * \brief Emits the signal to add a range of codepoints to the table.
 *
 * \param start Start of range.
 * \param end End of range.
 *
 * \see hideRange()
 * \see addRangeSignal()
 * \see TableModel::addRange()
 */
void TableController::addRange ( uint start, uint end )
{
    emit addRangeSignal ( start, end );
}

/**
 * \brief Emits the signal to adds a range of weights to the table.
 *
 * \param begin First/lowest collation.
 * \param end Last/highest collation.
 * \param flagstoint Defines which levels define the range. Use values of the enum
 *              collationMode.
 * \param collationnumbers Defines which collations define the range it letters
 *                         have multiple collations.
 *
 * \see addWeightRangeSignal()
 * \see TableModel::addWeightRange()
 */
void TableController::addWeightRange ( QStringList begin, QStringList end,
                                       TableModel::CollationModes flagstoint,
                                       QList<int> collationnumbers )
{
    emit addWeightRangeSignal ( begin, end, flagstoint, collationnumbers );
}

/**
 * \brief Emits the clear table signal.
 *
 * \see clearTable()
 * \see TableModel::clearVisible()
 */
void TableController::clearTable ()
{
    emit clear ();
}

/**
 * \brief Emit the signal to hide a range of codepoint from the table.
 *
 * \param start Start of range.
 * \param end End of range.
 *
 * \see addRange()
 * \see hideRangeSignal()
 * \see TableModel::hideRange()
 */
void TableController::hideRange ( uint start, uint end )
{
    emit hideRangeSignal ( start, end );
}

/**
 * \brief Emits the signal to select blocks.
 *
 * \param blocks The names of the blocks to select.
 * \param select If true the blocks with the given names are selected.
 *
 * \see selectBlocksSignal()
 * \see UCQBlockList::selectBlocks()
 */
void TableController::selectBlocks ( QStringList blocks, bool select )
{
    emit selectBlocksSignal ( blocks, select );
}
