/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *     																	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "LuaTable.h"
const char LuaTable::className[] = "LuaTable";

const Luna<LuaTable>::RegType
        LuaTable::Register[] = {
                                  { "show",     &LuaTable::show       },
                                  { "addRange", &LuaTable::addRange   },
                                  { "clear",    &LuaTable::clear      },
                                  { "addWeightRange",
                                            &LuaTable::addWeightRange },
                                  { "hideRange", &LuaTable::hideRange },
                                  { "hide",      &LuaTable::hide      },
                                  { 0, 0 } };
/**
 * \brief Sets the TableController reference to the object that is registered
 *        in Lua.
 *
 * \param L Lua interpreter state.
 */
LuaTable::LuaTable ( lua_State* L ) : LuaWrapper(L,"TableController")
{
    tableControl = ( TableController* ) lua_touserdata ( L, 1 );
}

/**
 * \brief Emtpy destructor (for Luna).
 */
LuaTable::~LuaTable ()
{
}

/**
 * \brief Shows blocks in the visible table.
 *
 * Shows the blocks with the given names. More blocks can be
 * add with more parameters. All arguments which are stored on the
 * Lua stack are read by this function.
 *
 * \param L Lua interpreter state.
 * \return Return the number of results.
 * \see TableController::selectBlocks()
 */
int LuaTable::show ( lua_State* L )
{
    int cnt = lua_gettop ( L );
    QStringList blocks;

    for ( int i = 2; i <= cnt; i++ )
    {
        blocks << QString ( lua_tostring(L, i) );
    }

    if ( blocks.isEmpty () )
    {
        blocks << tr ( "Show all" );
    }
    tableControl->selectBlocks ( blocks, true );

    return 0;
}

/**
 * \brief Hides the blocks.
 *
 * Hides the blocks with the given names. More blocks can be
 * removed with more parameters. All arguments that are stored on the
 * Lua stack, are read by this function.
 *
 * \param L Lua interpreter state.
 * \return Return the number of results.
 * \see TableController::selectBlocks()
 */
int LuaTable::hide ( lua_State* L )
{
    int cnt = lua_gettop ( L );
    QStringList blocks;

    for ( int i = 2; i <= cnt; i++ )
    {
        blocks << QString ( lua_tostring(L, i) );
    }
    tableControl->selectBlocks ( blocks, false );

    return 0;
}

/**
 * \brief Adds a range of codepoints to the visible table.
 *
 * Adds the letters with codepoints between or equal the beginning and the
 * end to the visible table. The beginning codepoint is stored on the Lua stack
 * at position 2 and the end codepoint is stored at position 3. After
 * getting the range, the addRange method from TableController will be called.
 *
 * \param L Lua interpreter state.
 * \return Return the number of results.
 * \see TableController::addRange()
 */
int LuaTable::addRange ( lua_State *L )
{
    bool number = true;
    uint a = QString ( lua_tostring(L,2) ).toUInt ( &number, 16 );
    if ( !number )
    {
        return -1;
    }

    uint b = QString ( lua_tostring(L,3) ).toUInt ( &number, 16 );
    if ( !number )
    {
        return -1;
    }
    tableControl->addRange ( a, b );
    return 0;
}

/**
 * \brief Removes a range of codepoints.
 *
 * Removes letters with codepoints between or equal the beginning and the
 * end from the visible table. The beginning codepoint is stored in the Lua
 * stack at position 2 and the end codepoint is stored at position 3. After
 * getting the range, the hideRange method from TableController will be called.
 *
 * \param L Lua interpreter state.
 * \return Return the number of results.
 * \see TableController::hideRange()
 */
int LuaTable::hideRange ( lua_State *L )
{
    bool number = true;
    uint a = QString ( lua_tostring(L,2) ).toUInt ( &number, 16 );
    if ( !number )
    {
        return -1;
    }

    uint b = QString ( lua_tostring(L,3) ).toUInt ( &number, 16 );
    if ( !number )
    {
        return -1;
    }
    tableControl->hideRange ( a, b );
    return 0;
}

/**
 * \brief Clears the table.
 *
 * Clears the visible table.
 *
 * \param L Lua interpreter state
 * \return Return the number of results.
 */
int LuaTable::clear ( lua_State *L )
{
    if ( L == NULL )
        return -1;
    tableControl->clearTable ();

    return 0;
}
/**
 * \brief Adds a collation range to the visible table.
 *
 * This function adds a collation range to the visible table.
 * The beginning list and end list of the ranges are stored on the Lua stack
 * at position 2 and 3. At position 4 are the flags. The flags are:\n
 * 's' The collation string is between or equal the begin and end string.\n
 * 'l1' Level 1 collation\n
 * 'l2' Level 2 collation\n
 * 'l3' Level 3 collation\n
 * 'l4' Level 4 collation\n
 * 'v' Variable collations (*)\n
 * 'n' Not variable collations (.)\n
 * At position 5 the indizes for the collations are stored. 0 is the
 * primary collation. 1 secondary.\n
 * If the beginning list and end list have not the same number of elements,
 * this function will return an error message to the Lua interpreter.
 * If the indices for collation are not a number, this function will return an
 * error message to the Lua interpreter.\n
 * If the range lists are smaller than the number of levels for search, this
 * function will return an error message to the Lua interpreter, too.\n
 * \n
 * More level flags could be set. Then the begin and end ranges will be separated
 * by semicolon.
 *
 *
 * \param L Lua interpreter.
 * \return Return the number of results.
 * \see TableController::addWeightRange().
 */
int LuaTable::addWeightRange ( lua_State *L )
{

    QStringList begin, end;
    begin = QString ( lua_tostring(L,2) ).split ( ';' );
    end = QString ( lua_tostring(L,3) ).split ( ';' );
    if ( begin.size () != end.size () )
    {
        lua_pushstring (
                L, QObject::tr (
                      "Begin and end have to be the same size." ).toAscii () );
        return 1;
    }

    QString flags ( lua_tostring(L,4) );
    QList<int> collationnumbers;

    if ( QString ( lua_tostring(L,5) ) != "" )
    {
        QStringList collations = QString ( lua_tostring(L,5) ).split ( ";" );

        for ( int i = 0; i < collations.size (); i++ )
        {
            bool number;
            int index = collations.at ( i ).toInt ( &number, 10 );
            if ( !number )
            {
                lua_pushstring (
                        L, QObject::tr (
                           "Index of collation is not a number" ).toAscii () );
                return 1;
            }
            collationnumbers << index;
        }
    }
    int flagstoint = 0;
    int lvlcnt = 0;
    if ( !flags.isEmpty () )
    {
        if ( flags.contains ( 's' ) )
        {
            flagstoint |= TableModel::COLLATION_STRING;
        }

        if ( flags.contains ( "l1" ) )
        {
            flagstoint |= TableModel::COLLATION_LEVEL1;
            lvlcnt++;
        }
        if ( flags.contains ( "l2" ) )
        {
            flagstoint |= TableModel::COLLATION_LEVEL2;
            lvlcnt++;
        }
        if ( flags.contains ( "l3" ) )
        {
            flagstoint |= TableModel::COLLATION_LEVEL3;
            lvlcnt++;
        }
        if ( flags.contains ( "l4" ) )
        {
            flagstoint |= TableModel::COLLATION_LEVEL4;
            lvlcnt++;
        }

        if ( flags.contains ( "v" ) )
        {
            flagstoint |= TableModel::COLLATION_VARIABLE;
        } else if ( flags.contains ( "n" ) )
        {
            flagstoint |= TableModel::COLLATION_NOTVARIABLE;
        }
        if ( lvlcnt == 0 )
        {
            flagstoint |= TableModel::COLLATION_STRING;
        }
    } else
    {
        flagstoint |= TableModel::COLLATION_STRING;
    }
    if ( lvlcnt > begin.size () )
    {
        lua_pushstring (
                   L, QObject::tr (
                           "Number of levels flags must be greater or same."
                           ).toAscii () );
        return 1;
    }

    tableControl->addWeightRange ( begin, end, (QFlags<
            TableModel::collationMode> ) flagstoint, collationnumbers );
    return 0;
}

